import { Schema, Table } from './schema';
import { AbstractDatabase } from './abstract-database';
import { EntityClass } from './query';
import * as node from './query';
/**
 * @brief Object => Entity instance (convert to its quoted ID);
 * string => field name
 */
export declare type ConvertableToNode = Object | node.ExpressionNode | string;
export declare let schema: Schema;
export declare function setDatabase(db: AbstractDatabase): void;
export declare function connect(): Promise<void>;
export declare function build(): Promise<void>;
export declare function dumpSchema(): Promise<void>;
export declare class Repository<T> {
    private entity;
    table: Table;
    constructor(entity: {
        new (): T;
    });
    query(): QueryBuilder<T>;
    findOne(extra?: Object): Promise<any>;
    findAll(extra?: Object[]): Promise<T[]>;
    insert(item: T): Promise<void>;
    delete(item: T): Promise<void>;
}
export declare class QueryBuilder<T> {
    entity: {
        new (): T;
    };
    private query;
    table: Table;
    constructor(entity: {
        new (): T;
    });
    findOne(extra?: Object): Promise<any>;
    findAll(extra?: Object[]): Promise<T[]>;
    insert(item: T): Promise<void>;
    delete(item?: T): Promise<void>;
    total(): Promise<number>;
    distinct(): this;
    limit(limit: number): this;
    offset(offset: number): this;
    sortBy(entity: EntityClass, field: string, order?: string): this;
    sortBy(field: string, order?: string): this;
    groupBy(entity: EntityClass, field: string): this;
    groupBy(field: string): this;
    alias(node: node.ExpressionNode, alias: string): void;
    where(...expr: node.ExpressionNode[]): this;
    andWhere(...expr: node.ExpressionNode[]): this;
    orWhere(...expr: node.ExpressionNode[]): this;
    and(...expr: node.ExpressionNode[]): node.ExpressionNode;
    or(...expr: node.ExpressionNode[]): node.ExpressionNode;
    private commonLogical(type, ...expr);
    eq(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    neq(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    lt(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    le(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    gt(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    ge(left: ConvertableToNode, right: ConvertableToNode): node.BinaryNode;
    private commonBinary(type, left, right);
    private convertToNode(param);
    sum(param: ConvertableToNode): node.CallNode;
    quote(value: any): node.LiteralNode;
    field(name: string): node.FieldNode;
}
